"""
   Copyright 2009 Glenn Johnson

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import wx, wx.html
from wx.lib.masked import NumCtrl
from wx.lib.pubsub import Publisher
from pkg_resources import resource_filename
from render import *

class MyHtmlWindow(wx.html.HtmlWindow):
    def __init__(self, parent, id):
	wx.html.HtmlWindow.__init__(self, parent, id)

    def OnLinkClicked(self, link):
	wx.LaunchDefaultBrowser(link.GetHref())

class EditDialog(wx.Dialog):
    def __init__(self, parent, ID, title, stock):
	wx.Dialog.__init__(self, parent, ID, title)
	data_symbol = {stock.symbol : stock.symbol}
	data_description = {stock.symbol : stock.description}
	vbox = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.FlexGridSizer(rows=4, cols=2, hgap=5, vgap=5)
	vbox.Add(vslot1)
	ticker_label = wx.StaticText(self, -1, "Symbol")
	self.ticker_textctrl = wx.TextCtrl(self, -1, stock.symbol)
	descript_label = wx.StaticText(self, -1, "Description")
	self.descript_textctrl = wx.TextCtrl(self, -1, stock.description,
		size=(400, -1))
	shares_label = wx.StaticText(self, -1, "Shares")
	self.shares_numctrl = NumCtrl(self, -1, stock.shares,
		fractionWidth=6, selectOnEntry=False)
	cpc_label = wx.StaticText(self, -1, "Core Portfolio Control")
	self.cpc_numctrl = NumCtrl(self, -1, stock.portfolio_control,
		fractionWidth=6, selectOnEntry=True)
	bpc_label = wx.StaticText(self, -1, "Buy Portfolio Control")
	self.bpc_numctrl = NumCtrl(self, -1, stock.buy_portfolio_control,
		fractionWidth=6, selectOnEntry=True)
	spc_label = wx.StaticText(self, -1, "Sell Portfolio Control")
	self.spc_numctrl = NumCtrl(self, -1, stock.sell_portfolio_control,
		fractionWidth=6, selectOnEntry=True)
	vslot1.AddMany([ticker_label, self.ticker_textctrl,
	    descript_label, self.descript_textctrl,
	    shares_label, self.shares_numctrl,
	    cpc_label, self.cpc_numctrl,
	    bpc_label, self.bpc_numctrl,
	    spc_label, self.spc_numctrl])

	sizer = self.CreateSeparatedButtonSizer(wx.OK|wx.CANCEL)
	vbox.Add(sizer, 1, wx.GROW)
	self.SetSizer(vbox)
	self.Fit()
	self.ticker_textctrl.SetFocus()

class DetailFrame(wx.Frame):
    def __init__(self, db, recs_html, config):
	config.SetPath('/Frames')
	width = config.ReadInt('detail_width', 600)
	height = config.ReadInt('detail_height', 300)
	wx.Frame.__init__(self, None, -1,
		"Portfolio Details",
		size=(width, height))
	self.panel = wx.Panel(self, -1)
	iconFile = resource_filename(__name__, 'icons/bar_chart.ico')
	ib = wx.IconBundle()
	ib.AddIconFromFile(iconFile, wx.BITMAP_TYPE_ANY)
	self.SetIcons(ib)

	D=db['portfolios']
	cash=db['cash']
	label = wx.StaticText(self.panel, -1,
		"Portfolio symbol")
	self.choice = wx.Choice(self.panel, -1, choices=sorted(D.keys()))
	self.choice.SetSelection(0)
	self.edit_btn = wx.Button(self.panel, -1, "&Edit")
	self.close_btn = wx.Button(self.panel, -1, "&Close")
	self.back_btn = wx.Button(self.panel, wx.ID_BACKWARD)
	self.next_btn = wx.Button(self.panel, wx.ID_FORWARD)

	self.details_html = MyHtmlWindow(self.panel, -1)
	if "gtk2" in wx.PlatformInfo:
	    self.details_html.SetStandardFonts()
	self.details_html.SetRelatedFrame(self, self.GetTitle() + ": %s")
	self.details_html.SetRelatedStatusBar(2)

	self.statusbar = self.CreateStatusBar(number=3)

	symbol = sorted(D.keys())[0]
	stock = D[symbol]
	self.render = Render()
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
	
	sizer = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.FlexGridSizer(rows=1, cols=6, hgap=5, vgap=5)
	vslot1.AddMany(
		[(label, 0, wx.ALIGN_CENTER), 
		    (self.choice, 0, wx.ALIGN_CENTER),
		    (self.back_btn, 0, wx.ALIGN_CENTER),
		    (self.next_btn, 0, wx.ALIGN_CENTER),
		    (self.edit_btn, 0, wx.ALIGN_CENTER),
		    (self.close_btn, 0, wx.ALIGN_CENTER)])
	sizer.Add(vslot1)
	sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
	sizer.Add(sizer_2, 1, wx.GROW)
	sizer_2.Add(self.details_html, 1, wx.GROW | wx.RIGHT, 5)
	staticbox = wx.StaticBox(self.panel, -1, "Parameters")
	sb_sizer = wx.StaticBoxSizer(staticbox, wx.VERTICAL)
	sizer_2.Add(sb_sizer)
	
	br_label = wx.StaticText(self.panel, -1, "Buy Resistance")
	self.br_sc = wx.SpinCtrl(self.panel, -1, initial=stock.buy_resistance)
	
	sr_label = wx.StaticText(self.panel, -1, "Sell Resistance")
	self.sr_sc = wx.SpinCtrl(self.panel, -1, initial=stock.sell_resistance)

	mb_label = wx.StaticText(self.panel, -1, "Minimum Buy")
	self.mb_sc = wx.SpinCtrl(self.panel, -1, initial=stock.min_buy)

	ms_label = wx.StaticText(self.panel, -1, "Minimum Sell")
	self.ms_sc = wx.SpinCtrl(self.panel, -1, initial=stock.min_sell)

	sb_sizer.AddMany([(br_label, 0, wx.TOP, 5),
	    (self.br_sc, 0, wx.BOTTOM, 5),
	    sr_label, (self.sr_sc, 0, wx.BOTTOM, 5),
	    mb_label, (self.mb_sc, 0, wx.BOTTOM, 5),
	    ms_label, (self.ms_sc, 0, wx.BOTTOM, 5)])

	self.panel.SetSizer(sizer)
	self.panel.Layout()
	self.SetMinSize(sizer.GetMinSize())
	self.details_html.SetFocus()

	self.Bind(wx.EVT_BUTTON, self.OnClose, self.close_btn)

	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnBR(event, db, recs_html), 
		self.br_sc)
	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnSR(event, db, recs_html),
		self.sr_sc)
	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnMB(event, db, recs_html),
		self.mb_sc)
	self.Bind(wx.EVT_SPINCTRL, 
		lambda event: self.OnMS(event, db, recs_html),
		self.ms_sc)
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnEdit(event, db, recs_html),
		self.edit_btn)
	self.Bind(wx.EVT_CHOICE, 
		lambda event: self.OnChoice(event, db), 
		self.choice)
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnNext(event, db),
		self.next_btn)
	self.Bind(wx.EVT_BUTTON, 
		lambda event: self.OnBack(event, db),
		self.back_btn)
	Publisher().subscribe(self.OnReceiveMessage, 'update.cash_level')
	Publisher().subscribe(self.OnReceiveMessage, 'update.prices')

    def OnReceiveMessage(self, message):
	cash = message.data['cash']
	D = message.data['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnBR(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.buy_resistance = float(self.br_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnSR(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.sell_resistance = float(self.sr_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
    
    def OnMB(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.min_buy = float(self.mb_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
    
    def OnMS(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	stock.min_sell = float(self.ms_sc.GetValue())
	stock.calc(cash.level, db['delay'])
	db['portfolios'] = D
	db.sync()
	self.render.recs(recs_html, db)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnEdit(self, event, db, recs_html):
	cash = db['cash']
	D = db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = D[symbol]
	edit_d = EditDialog(self, -1, 'Edit', stock)
	edit_d.shares_numctrl.SetMin(0.000001)
	edit_d.shares_numctrl.SetLimited(True)
	result = edit_d.ShowModal()
	if result == wx.ID_OK:
	    stock.symbol = edit_d.ticker_textctrl.GetValue()
	    stock.description = edit_d.descript_textctrl.GetValue()
	    stock.shares = edit_d.shares_numctrl.GetValue()
	    stock.portfolio_control = edit_d.cpc_numctrl.GetValue()
	    stock.buy_portfolio_control = edit_d.bpc_numctrl.GetValue()
	    stock.sell_portfolio_control = edit_d.spc_numctrl.GetValue()
	    stock.calc(cash.level, db['delay'])
	    db['portfolios'] = D
	    db.sync()
	    edit_d.Destroy()
	    self.render.recs(recs_html, db)
	    self.render.details(self.details_html, stock, cash)
	    self.statusbar.SetStatusText(
		    "Buy Resistance (%%): %.2f" 
		    % stock.scaled_buy_resistance, 0)
	    self.statusbar.SetStatusText(
		    "Sell Resistance (%%): %.2f"
		    % stock.scaled_sell_resistance, 1)
	    return
	else:
	    edit_d.Destroy()

    def OnNext(self, event, db):
	cash = db['cash']
	D = db['portfolios']
	count=self.choice.GetCount()
	index=self.choice.GetSelection()
	if index < (count - 1):
	    index += 1
	else:
	    return
	self.choice.SetSelection(index)
	symbol = self.choice.GetString(index)
	stock = D[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnBack(self, event, db):
	cash = db['cash']
	D = db['portfolios']
	count=self.choice.GetCount()
	index=self.choice.GetSelection()
	if index > 0:
	    index -= 1
	else:
	    return
	self.choice.SetSelection(index)
	symbol = self.choice.GetString(index)
	stock = D[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnClose(self, event):
	width, height = self.GetSize()
	Publisher().sendMessage('details.size', (width, height))
	self.Close()

    def OnChoice(self, event, db):
	cash = db['cash']
	D = db['portfolios']
	symbol = event.GetString()
	stock = D[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

